//--------------------------------------------------------------------------------------------------------------------------------------------------------------

#ifndef SortedStringGridH
#define SortedStringGridH
//--------------------------------------------------------------------------------------------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <Grids.hpp>
//--------------------------------------------------------------------------------------------------------------------------------------------------------------
enum TGridSortEventType {gsAdd, gsRemove, gsReplace, gsToggle } ;
enum TSortedGridGlyphType {sgAuto, sgLarge, sgMedium, sgSmall, sgSmallEmbossed, sgNone } ;
//--------------------------------------------------------------------------------------------------------------------------------------------------------------
typedef void __fastcall (__closure *TCustomCompareEvent)(TComponent *Sender
	, AnsiString String1, int ColumnIndex1, int RowIndex1
	, AnsiString String2, int ColumnIndex2, int RowIndex2
	, int &Result ) ;

typedef void __fastcall (__closure *TGridSelectEvent)(TComponent *Sender, int Index, bool &CanProceed) ;

typedef void __fastcall (__closure *TGridSortEvent)(TComponent *Sender, int Index, TGridSortEventType EventType, bool &CanProceed) ;
//--------------------------------------------------------------------------------------------------------------------------------------------------------------
class PACKAGE TSortedStringGrid : public TStringGrid
{
private:
/*				 _   _           _   ___  __
 *				|_) |_) | \  /  /_|   |  |_
 *				|   | \ |  \/  /  |   |  |__
 */
//______Member Variables________________________________________________________________________________________________________________________________________
	int mSortColumn, mSortRow ;
	bool mCanDrag, mSizeChange, mColumnToggled, mRowToggled ;
	TStringList *mSortList, *mSortListArray ;

//______Property Fields_________________________________________________________________________________________________________________________________________
	TColor mSortedColor ;
	TColor mSelectedColor ;
	TFont *mFixedFont ;
	TSortedGridGlyphType mRowGlyph;
	TSortedGridGlyphType mColumnGlyph;
	bool mToggle;
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	TCustomCompareEvent FOnCustomCompare ;
	TGridSelectEvent FOnSelectColumn ;
	TGridSelectEvent FOnSelectRow ;
	TGridSelectEvent FOnDeselectColumn ;
	TGridSelectEvent FOnDeselectRow ;
	TGridSortEvent FOnSortColumns ;
	TGridSortEvent FOnSortRows ;

//______Property Get/Set Methods________________________________________________________________________________________________________________________________
	void __fastcall SetSortedColor(TColor aValue) ;
	void __fastcall SetSelectedColor(TColor aValue) ;
	void __fastcall SetFixedFont(TFont *aValue) ;
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	unsigned int __fastcall GetColumnId(int aColumnIndex) ;
	void __fastcall SetColumnGlyph(TSortedGridGlyphType aValue);
	void __fastcall SetColumnId(int aColumnIndex, unsigned int aValue) ;
	unsigned int __fastcall GetRowId(int aRowIndex) ;
	void __fastcall SetRowGlyph(TSortedGridGlyphType aValue);
	void __fastcall SetRowId(int aRowIndex, unsigned int aValue) ;


//______Friends_________________________________________________________________________________________________________________________________________________
	friend int __fastcall RowSortCompare(TStringList* List, int Index1, int Index2) ;
	friend int __fastcall ColumnSortCompare(TStringList* List, int Index1, int Index2) ;


protected:
/*				 _   _   _  ___  __   __  ___  __   __
 *				|_) |_) | |  |  |_   |     |  |_   |  \
 *				|   | \ |_|  |  |__  |__   |  |__  |__/
 */
//______Member Variables________________________________________________________________________________________________________________________________________
	bool mSorting ;
	int mSortColumns[32], mSortRows[32] ;
	int mSortColumnCount, mSortRowCount ;
	unsigned int *mColumnAttributes, *mRowAttributes ;

//______Method Specifications___________________________________________________________________________________________________________________________________
	int __fastcall GetSortColumnIndex(int aSortIndex) ;
	int __fastcall GetSortRowIndex(int aSortIndex) ;
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	virtual void __fastcall DrawCell(int ACol, int ARow, const Windows::TRect & ARect, Grids::TGridDrawState AState) ;
	DYNAMIC void __fastcall MouseDown(Controls::TMouseButton Button, Classes::TShiftState Shift, int X, int Y) ;
	DYNAMIC void __fastcall MouseUp(Controls::TMouseButton Button, Classes::TShiftState Shift, int X, int Y) ;
	DYNAMIC bool __fastcall BeginColumnDrag(int &Origin, int &Destination, const Windows::TPoint &MousePt) ;
	DYNAMIC bool __fastcall BeginRowDrag(int &Origin, int &Destination, const Windows::TPoint &MousePt) ;
	DYNAMIC void __fastcall ColumnMoved(int FromIndex, int ToIndex) ;
	DYNAMIC void __fastcall RowMoved(int FromIndex, int ToIndex) ;
	DYNAMIC void __fastcall ColWidthsChanged(void);
	DYNAMIC void __fastcall RowHeightsChanged(void);
	DYNAMIC void __fastcall SizeChanged(int OldColCount, int OldRowCount) ;
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	virtual int __fastcall DoCompare( AnsiString aString1, int aColumnIndex1, int aRowIndex1, AnsiString aString2, int aColumnIndex2, int aRowIndex2 ) ;


public:
/*				 _       _         __
 *				|_) | | |_) |   | |
 *				|   |_| |_) |__ | |__
 */
//______Constructor And Destructor______________________________________________________________________________________________________________________________
	__fastcall TSortedStringGrid(TComponent* Owner) ;
	virtual __fastcall ~TSortedStringGrid() ;

//______Property Definitions____________________________________________________________________________________________________________________________________
	__property unsigned int ColumnId[int aColumnIndex]  = { read=GetColumnId, write=SetColumnId } ;
	__property unsigned int RowId[int aRowIndex]  = { read=GetRowId, write=SetRowId } ;
	__property unsigned int SortColumnIndex[int aSortIndex]  = { read=GetSortColumnIndex } ;
	__property unsigned int SortRowIndex[int aSortIndex]  = { read=GetSortRowIndex } ;
	__property int SortColumnCount  = { read=mSortColumnCount } ;
	__property int SortRowCount  = { read=mSortRowCount } ;

//______Method Specifications___________________________________________________________________________________________________________________________________
	bool __fastcall GetColumnAttributes(int aColumnIndex, bool &aIsSorted, bool &aIsAscending, bool &aIsSelected) ;
	bool __fastcall SetColumnAttributes(int aColumnIndex, bool aIsSorted, bool aIsAscending, bool aIsSelected) ;
	bool __fastcall InsertColumn(int aColumnIndex) ;
	void __fastcall DeleteColumn(int aColumnIndex) ;
	void __fastcall MoveColumn(int aFromIndex, int aToIndex) ;
	bool __fastcall SortColumns() ;
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	bool __fastcall GetRowAttributes(int aRowIndex, bool &aIsSorted, bool &aIsAscending, bool &aIsSelected) ;
	bool __fastcall SetRowAttributes(int aRowIndex, bool aIsSorted, bool aIsAscending, bool aIsSelected) ;
	bool __fastcall InsertRow(int aRowIndex) ;
	void __fastcall DeleteRow(int aRowIndex) ;
	void __fastcall MoveRow(int aFromIndex, int aToIndex) ;
	bool __fastcall SortRows() ;


__published:
/*				 _       _         __        __  __
 *				|_) | | |_) |   | |__  |__| |_  |  \
 *				|   |_| |_) |__ |  __| |  | |__ |__/
 */
//______Property Definitions____________________________________________________________________________________________________________________________________
	__property TFont * FixedFont  = { read=mFixedFont, write=SetFixedFont } ;
	__property TColor SelectedColor  = { read=mSelectedColor, write=SetSelectedColor } ;
	__property TColor SortedColor  = { read=mSortedColor, write=SetSortedColor } ;
	__property TSortedGridGlyphType RowGlyph  = { read=mRowGlyph, write=SetRowGlyph };
	__property TSortedGridGlyphType ColumnGlyph  = { read=mColumnGlyph, write=SetColumnGlyph };
	__property bool Toggle  = { read=mToggle, write=mToggle };

//______Event Definitions_______________________________________________________________________________________________________________________________________
	__property TCustomCompareEvent OnCustomCompare = { read=FOnCustomCompare, write=FOnCustomCompare } ;
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	__property TGridSortEvent OnSortColumns  = { read=FOnSortColumns, write=FOnSortColumns };
	__property TGridSelectEvent OnSelectColumn  = { read=FOnSelectColumn, write=FOnSelectColumn };
	__property TGridSelectEvent OnDeselectColumn  = { read=FOnDeselectColumn, write=FOnDeselectColumn };
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	__property TGridSortEvent OnSortRows  = { read=FOnSortRows, write=FOnSortRows };
	__property TGridSelectEvent OnSelectRow  = { read=FOnSelectRow, write=FOnSelectRow };
	__property TGridSelectEvent OnDeselectRow  = { read=FOnDeselectRow, write=FOnDeselectRow };
};
//--------------------------------------------------------------------------------------------------------------------------------------------------------------
#endif
